// (c) Copyright 2012 Cloudera, Inc. All rights reserved.
define([
			 "cloudera/Util",
			 "cloudera/chart/TimeLabelUpdater",
			 "cloudera/chart/TimeRange"
], function(Util, TimeLabelUpdater, TimeRange) {

	describe("TimeLabelUpdater Tests", function() {

		var labelUpdater, oldTimezoneOffset = Util.getServerTimezoneOffset(),
		oldTimezoneDisplayName = Util.getServerTimezoneDisplayName(),
		MS_IN_ONE_HOUR = 60 * 60 * 1000;

		beforeEach(function() {
			labelUpdater = new TimeLabelUpdater();
			$rangeLabel = $("<div>").addClass("selectedTimeRangeLabel");
			$markerLabel = $("<div>").addClass("selectedTimeEndTimeLabel");
			$("body").append($rangeLabel).append($markerLabel);
			Util.setServerTimezoneOffset(0);
			Util.setServerTimezoneDisplayName("UTC");
			spyOn(Util, "getServerNow").andReturn(new Date(1000));
		});

		afterEach(function() {
			labelUpdater.unsubscribe();
			$(".selectedTimeRangeLabel").remove();
			$(".selectedTimeEndTimeLabel").remove();
			Util.setServerTimezoneOffset(oldTimezoneOffset);
			Util.setServerTimezoneDisplayName(oldTimezoneDisplayName);
		});

		it("should update the time selection label.", function() {
			var timeRange = new TimeRange(new Date(10 * MS_IN_ONE_HOUR), new Date(20 * MS_IN_ONE_HOUR));
			$.publish("timeSelectionChanged", [timeRange, true]);
			var actual = $(".selectedTimeRangeLabel").html();
			var expected = '(January 1 1970, 9:00 AM - 7:00 PM UTC)<i class="icon-time" title=""></i>';
			expect(actual).toEqual(expected);
		});

		it("should update the time selection label when the start/end date differ.", function() {
			var timeRange = new TimeRange(new Date(10 * MS_IN_ONE_HOUR), new Date(50 * MS_IN_ONE_HOUR));
			$.publish("timeSelectionChanged", [timeRange, false]);
			var actual = $(".selectedTimeRangeLabel").html();
			var expected = '(January 1 1970, 9:00 AM - January 3 1970, 1:00 AM UTC)<i class="icon-time" title=""></i>';
			expect(actual).toEqual(expected);
		});

		it("should update the time marker label.", function() {
			var markerDate = new Date(30 * MS_IN_ONE_HOUR);
			$.publish("markerDateChanged", [markerDate, false]);
			var actual = $(".selectedTimeEndTimeLabel").html();
			var expected = '(January 2 1970, 5:00 AM UTC)<span class="instantMarkerSmallIcon" title=""></span>';
			expect(actual).toEqual(expected);
		});

		it("should update the time marker label in current mode.", function() {
			var markerDate = new Date(30 * MS_IN_ONE_HOUR);
			$.publish("markerDateChanged", [markerDate, true]);
			var actual = $(".selectedTimeEndTimeLabel").html();
			var expected = '(Current)<span class="instantMarkerSmallIcon" title=""></span>';
			expect(actual).toEqual(expected);
		});
	});
});
