// Copyright (c) 2011-2012 Cloudera, Inc.  All rights reserved.
define([
	"cloudera/Util"
], function(Util) {

return function(options) {

	var showSelectedTimeRange = false;
	var currentRange;

	var _handleTimeChange = function(range){

		var url, timeRange, urlParams;

		// Every time this method is called,
		// store the currentRange
		if (range) {
			currentRange = range;
		}

		if (showSelectedTimeRange) {
			if (range) {
				options.urlParams.startTime = range.startDate.getTime();
				options.urlParams.endTime = range.endDate.getTime();
			}

			url = options.url + "?" + $.param(options.urlParams);

		} else {

			timeRange = {
				startTime: -1,
				endTime: -1
			};
			urlParams = $.extend({}, options, timeRange);
			url = options.url + "?" + $.param(urlParams);
		}

		$.get(url, function(response) {
			try {
				$(options.containerSelector).html(Util.filterError(response));
				checkMaximumCount($(options.outerContainerSelector));
			} catch (ex) {
				console.log(ex);
			}
		});
	};

	// Initially, the start/end values from options,
	// subsequently, they come from a callback function,
	// which provides a range parameter.
	var initialRange = {
		startDate: new Date(options.urlParams.startTime),
		endDate: new Date(options.urlParams.endTime)
	};

	// Update the initial value on ready.
	_handleTimeChange(initialRange);
	jQuery.subscribe("timeSelectionChanged", _handleTimeChange);

  // This page should never auto refresh.
	jQuery.publish("pauseAutoRefresh");

	var getAllCheckBoxes = function($elem) {
		return $elem.find("input[type=checkbox]");
	};

	/**
	 * Returns the number of checkboxes that are selected.
	 */
	var getCurrentCount = function($elem) {
		return getAllCheckBoxes($elem).filter(":checked").length;
	};

	/**
	 * Only allow 2 revisions to be selected.
	 */
	var checkMaximumCount = function($elem) {
		var currentCount = getCurrentCount($elem);
		var disableCompareButton = true;
		if (currentCount >= 2) {
			getAllCheckBoxes($elem).not(":checked").prop('disabled', true);
			if (currentCount === 2) {
				disableCompareButton = false;
			}
		} else {
			getAllCheckBoxes($elem).removeAttr('disabled');
		}
		// Enable the Compare button.
		var $compareBtn = $elem.find("button[value=compare]");
		if (disableCompareButton) {
			$compareBtn.attr("disabled", true);
		} else {
			$compareBtn.removeAttr("disabled");
		}
	};

	var onClickHandler = function(evt) {
		var $target = $(evt.target);
		var $tr = $target.closest("tr");
		var $checkbox = $tr.find("input[type=checkbox]");
		if (!$target.is($checkbox) && !$checkbox.prop("disabled")) {
			// toggle the checkbox if it is not disabled.
			$checkbox.prop('checked', !$checkbox.prop('checked'));
		}
		if ($checkbox.length > 0) {
			checkMaximumCount($(options.outerContainerSelector));
		}
	};

	var toggleShowSelectedTimeRange = function(evt, newShowSelectedTimeRange, range) {
		evt.preventDefault();

		if (showSelectedTimeRange !== newShowSelectedTimeRange) {
			showSelectedTimeRange = newShowSelectedTimeRange;
			// passing undefined is OK because it is not used.
			_handleTimeChange(range);
			if (showSelectedTimeRange) {
				// Find the showAll button, and toggle the selected state.
				// notSelected is required because otherwise
				// the CSS is not applied correctly.
				$(options.outerContainerSelector).find(".showAll")
					.addClass("notSelected").removeClass("selected")
				.end().find(".showSelectedTimeRange")
					.addClass("selected").removeClass("notSelected");
			} else {
				$(options.outerContainerSelector).find(".showSelectedTimeRange")
					.addClass("notSelected").removeClass("selected")
				.end().find(".showAll").addClass("selected").removeClass("notSelected");
			}
		} else {
			return false;
		}
	};

	var onShowAll = function(evt) {
		return toggleShowSelectedTimeRange(evt, false, undefined);
	};

	var onShowSelectedTimeRange = function(evt) {
		return toggleShowSelectedTimeRange(evt, true, currentRange);
	};

	setTimeout(function(){
		checkMaximumCount($(options.outerContainerSelector));
	}, 100);

	var $outerContainerSelector = $(options.outerContainerSelector);

	$outerContainerSelector.click(onClickHandler).find(".showAll a")
		.click(onShowAll)
	.end().find(".showSelectedTimeRange a")
		.click(onShowSelectedTimeRange);

	// otherwise this DOM variable would
	// be visible in all the private functions of this closure,
	// may leak memory.
	$outerContainerSelector = null;
};

});
