// Copyright (c) 2011-2012 Cloudera, Inc.  All rights reserved.
define([], function() {
	$.fn.FixedTableHeader = function( method ) {

		//PRIVATE

		/*
		 * Calculates width of scrollbar by drawing
		 * two divs off the page.  Because this is
		 * relatively heavy, we cache the result.
		 */
		var _scrollbarWidth;
		var scrollbarWidth = function() {
			if (_scrollbarWidth !== undefined) {
				return _scrollbarWidth;
			}
			var testOuter = $('<div />');
			var testInner = $('<div />');

			testOuter
				.css('width', '100px')
				.css('height', '100px')
				.css('overflow', 'hidden')
				.css('position', 'absolute')
				.css('top', '-1000px')
				.css('left', '-1000px');

			testInner
				.css('height', '200px');

			testOuter.html(testInner);

			$('body').append(testOuter);

			var noScrollWidth = $(testOuter, 'div')[0].clientWidth;
			testOuter.css('overflow-y', 'scroll');
			var scrollWidth = $(testOuter, 'div')[0].clientWidth;
			testOuter.remove();

			_scrollbarWidth = noScrollWidth - scrollWidth;
			return _scrollbarWidth;

		};

		/*
		 * Returns array containing widths of cells in
		 * $table at given row
		 */
		var getHeaderCellWidths = function ($table, row) {
			var widths = [];
			var cells = $.makeArray($table.find('thead tr:eq(' + row + ') th'));

			var i;
			for (i = 0; i < cells.length; i += 1) {
				widths.push($(cells[i]).width());
			}
			return widths;
		};

		/*
		 * Sets cell widths in $table at the given row
		 * to the values in the widths array.  Assumes
		 * that widths equals the number of cells
		 * in the indicated row
		 */
		var setHeaderCellWidths = function ($table, row, widths) {
			var cells = $table.find('thead tr:eq(' + row + ') th');
			var i;
			for (i = 0; i < cells.length; i += 1) {
				$(cells[i])
					.width(widths[i])
					.css('min-width', widths[i]);
			}
		};

		/*
		 * Sets the width of the header table parent
		 * element to that of the original table parent
		 * element.  Takes into account the presence of a scrollbar.
		 */
		var syncHeaderTotalWidth = function($table, $headerTable) {
			var parent = $table.parent()[0];
			var newWidth = $table.parent().width();

			if (parent.scrollHeight > parent.clientHeight) {
				newWidth -= scrollbarWidth();
			}
			$headerTable.parent().width(newWidth);
		};

		/*
		 * Sets the width of the header table cells at
		 * a given row to the width of the header cells
		 * in the original table at the same row.
		 */
		var syncHeaderCellWidths = function($table, $headerTable, row) {
			var widths = getHeaderCellWidths($table, row);
			setHeaderCellWidths($headerTable, row, widths);
		};

		/*
		 * Sync both header cell width and total header width
		 */
		var _syncHeader = function ($table, $headerTable, row) {
			syncHeaderCellWidths($table, $headerTable, row);
			syncHeaderTotalWidth($table, $headerTable, $table);

		};

		var POLLING_INTERVAL = 500;

		/*
		 * Uses a polling loop to detect
		 * when an elements width has changed
		 */
		$.event.special.widthChange = {

			setup : function () {
				$this = $(this);
				var oldWidth = $this.width();

				var interval = setInterval(function () {
					if ($this.width() !== oldWidth) {
						oldWidth = $this.width();
						$this.triggerHandler('widthChange');
					}
				}, POLLING_INTERVAL);

				$this.data("widthChange.interval", interval);
			},

			teardown : function () {
				clearInterval($(this).data("widthChange.interval"));
			}

		};

		/*
		 * Uses a polling loop to detect
		 * when an elements height has changed
		 */
		$.event.special.heightChange = {

			setup : function () {
				$this = $(this);
				oldHeight = $this.height();
				var interval = setInterval(function () {
					if ($this.height() !== oldHeight) {
						oldHeight = $this.height();
						$this.triggerHandler('heightChange');
					}
				}, POLLING_INTERVAL);

				$this.data("heightChange.interval", interval);
			},

			teardown : function () {
				clearInterval($(this).data("heightChange.interval"));
			}
		};

		// PUBLIC

		var methods = {

			init : function (options) {

				var o = $.extend({
					// row in thead to use as reference for cell widths
					headerRow : 0
				}, options);

				return this.each(function(method) {

					var $table = $(this);
					$table.data('headerRow', o.headerRow);

					// Record original widths
					//
					// KNOWN ISSUE: Mozilla records non-rounded
					// calculated widths while Webkit rounds before storage.
					// This can cause 1px errors in FF.  However, because
					// we don't have vertical borders between table body
					// cells, this isn't a huge deal.
					var widths = getHeaderCellWidths($table, o.headerRow);

					// Create new header table.  Set overflow
					// to hidden to hide scrollbar.
					var $headerTable = $table.clone();
					$headerTable.find('tbody').remove();
					$headerTableCont = $('<div />')
						.append($headerTable)
						.css('overflow', 'hidden');

					// Sync both now and when the container width changes.
					_syncHeader($table, $headerTable, o.headerRow);
					$table.parent().bind('widthChange', function (e) {
						_syncHeader($table, $headerTable, o.headerRow);
					});

					// Add and remove scrollbar spacer on header
					// as necessary.
					$table.parent().bind('heightChange', function (e) {
						syncHeaderTotalWidth($table, $headerTable);
					});

					// Scroll new header horizontally in step
					// with new table.
					$table.parent().scroll(function (e) {
						$headerTableCont.scrollLeft($table.parent().scrollLeft());
					});


					// Insert header table and hide original header
					// behind it.
					$headerTableCont.insertBefore($table.parent());
					$table.css('margin-top', $headerTableCont.outerHeight(true) * -1);

				});

			},

			/*
			 * Public method to force header sync when
			 * necessary (i.e. when we update the content
			 * of a cell).
			 */
			syncHeader : function() {
				return this.each(function() {
					$table = $(this);
					if (!$table.parent().prev()) {
						return;
					}
					$headerTable = $table.parent().prev().find('table');
					_syncHeader($table, $headerTable, $table.data('headerRow'));
				});
			}
		};

		//ROUTE

		if (methods[method]) {
			return methods[method].apply(this, Array.prototype.slice.call(arguments,1));
		} else if (typeof method === 'object' || !method) {
			return methods.init.apply(this, arguments);
		} else {
			$.error('Method ' + method + ' does not exist.');
		}


	};
});
